@@ -73,7 +73,7 @@ QCString includeStatement(SrcLangExt lang,IncludeKind kind)
 QCString includeOpen(SrcLangExt lang,IncludeKind kind)
 {
   if (lang==SrcLangExt::Java || kind==IncludeKind::ImportModule) return "";
-  if (kind & IncludeKind_LocalMask)
+  if ((kind & IncludeKind_LocalMask) || (lang==SrcLangExt::IDL))
   {
     return "\"";
   }
@@ -85,7 +85,9 @@ QCString includeOpen(SrcLangExt lang,IncludeKind kind)
 
 QCString includeClose(SrcLangExt lang,IncludeKind kind)
 {
-  if (lang==SrcLangExt::Java || lang==SrcLangExt::IDL) return ";";
+  if (lang==SrcLangExt::IDL) return "\";";
+  else if (lang==SrcLangExt::Java) return ";";
+
   switch (kind)
   {
     case IncludeKind::ImportLocal:       return "\";";
@@ -150,7 +152,7 @@ class FileDefImpl : public DefinitionMixin<FileDef>
     DirDef *getDirDef() const override       { return m_dir; }
     ModuleDef *getModuleDef() const override { return m_moduleDef; }
     const LinkedRefMap<NamespaceDef> &getUsedNamespaces() const override;
-    const LinkedRefMap<ClassDef> &getUsedClasses() const override  { return m_usingDeclList; }
+    const LinkedRefMap<const Definition> &getUsedDefinitions() const override  { return m_usingDeclList; }
     const IncludeInfoList &includeFileList() const override    { return m_includeList; }
     const IncludeInfoList &includedByFileList() const override { return m_includedByList; }
     void getAllIncludeFilesRecursively(StringVector &incFiles) const override;
@@ -179,6 +181,7 @@ class FileDefImpl : public DefinitionMixin<FileDef>
     void parseSource(ClangTUParser *clangParser) override;
     void setDiskName(const QCString &name) override;
     void insertMember(MemberDef *md) override;
+    void removeMember(MemberDef *md) override;
     void insertClass(ClassDef *cd) override;
     void insertConcept(ConceptDef *cd) override;
     void insertNamespace(NamespaceDef *nd) override;
@@ -186,7 +189,7 @@ class FileDefImpl : public DefinitionMixin<FileDef>
     void setDirDef(DirDef *dd) override { m_dir=dd; }
     void setModuleDef(ModuleDef *mod) override { m_moduleDef=mod; }
     void addUsingDirective(NamespaceDef *nd) override;
-    void addUsingDeclaration(ClassDef *cd) override;
+    void addUsingDeclaration(const Definition *d) override;
     void combineUsingRelations() override;
     bool generateSourceFile() const override;
     void sortMemberLists() override;
@@ -227,13 +230,14 @@ class FileDefImpl : public DefinitionMixin<FileDef>
     void writeDetailedDescription(OutputList &ol,const QCString &title);
     void writeBriefDescription(OutputList &ol);
     void writeClassesToTagFile(TextStream &t,const ClassLinkedRefMap &list);
+    void removeMemberFromList(MemberListType lt,MemberDef *md);
 
     IncludeInfoMap        m_includeMap;
     IncludeInfoList       m_includeList;
     IncludeInfoMap        m_includedByMap;
     IncludeInfoList       m_includedByList;
     LinkedRefMap<NamespaceDef> m_usingDirList;
-    LinkedRefMap<ClassDef> m_usingDeclList;
+    LinkedRefMap<const Definition> m_usingDeclList;
     QCString              m_path;
     QCString              m_filePath;
     QCString              m_inclDepFileName;
@@ -324,7 +328,7 @@ void FileDefImpl::setDiskName(const QCString &name)
 /*! Compute the HTML anchor names for all members in the class */
 void FileDefImpl::computeAnchors()
 {
-  MemberList *ml = getMemberList(MemberListType_allMembersList);
+  MemberList *ml = getMemberList(MemberListType::AllMembersList());
   if (ml) ml->setAnchors();
 }
 
@@ -349,7 +353,7 @@ void FileDefImpl::findSectionsInDocumentation()
 
   for (auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_declarationLists)
+    if (ml->listType().isDeclaration())
     {
       ml->findSectionsInDocumentation(this);
     }
@@ -437,7 +441,7 @@ void FileDefImpl::writeTagFile(TextStream &tagFile)
       case LayoutDocEntry::MemberDecl:
         {
           const LayoutDocEntryMemberDecl *lmd = dynamic_cast<const LayoutDocEntryMemberDecl*>(lde.get());
-          if (lmd)
+          if (lmd && lmd->visible())
           {
             MemberList * ml = getMemberList(lmd->type);
             if (ml)
@@ -823,12 +827,12 @@ void FileDefImpl::writeSummaryLinks(OutputList &ol) const
     else if (lde->kind()==LayoutDocEntry::MemberDecl)
     {
       const LayoutDocEntryMemberDecl *lmd = dynamic_cast<const LayoutDocEntryMemberDecl*>(lde.get());
-      if (lmd)
+      if (lmd && lmd->visible())
       {
         MemberList * ml = getMemberList(lmd->type);
         if (ml && ml->declVisible())
         {
-          ol.writeSummaryLink(QCString(),MemberList::listTypeAsString(ml->listType()),lmd->title(lang),first);
+          ol.writeSummaryLink(QCString(),ml->listType().toLabel(),lmd->title(lang),first);
           first=FALSE;
         }
       }
@@ -963,7 +967,7 @@ void FileDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDecl:
         {
           const LayoutDocEntryMemberDecl *lmd = dynamic_cast<const LayoutDocEntryMemberDecl*>(lde.get());
-          if (lmd) writeMemberDeclarations(ol,lmd->type,lmd->title(lang));
+          if (lmd && lmd->visible()) writeMemberDeclarations(ol,lmd->type,lmd->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDeclEnd:
@@ -981,7 +985,7 @@ void FileDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::MemberDef:
         {
           const LayoutDocEntryMemberDef *lmd = dynamic_cast<const LayoutDocEntryMemberDef*>(lde.get());
-          if (lmd) writeMemberDocumentation(ol,lmd->type,lmd->title(lang));
+          if (lmd && lmd->visible()) writeMemberDocumentation(ol,lmd->type,lmd->title(lang));
         }
         break;
       case LayoutDocEntry::MemberDefEnd:
@@ -1023,8 +1027,8 @@ void FileDefImpl::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::DirSubDirs:
       case LayoutDocEntry::DirFiles:
       case LayoutDocEntry::DirGraph:
-        err("Internal inconsistency: member %d should not be part of "
-            "LayoutDocManager::File entry list\n",lde->kind());
+        err("Internal inconsistency: member '%s' should not be part of "
+            "LayoutDocManager::File entry list\n",qPrint(lde->entryToString()));
         break;
     }
   }
@@ -1037,7 +1041,7 @@ void FileDefImpl::writeDocumentation(OutputList &ol)
 
   if (Config_getBool(SEPARATE_MEMBER_PAGES))
   {
-    MemberList *ml = getMemberList(MemberListType_allMembersList);
+    MemberList *ml = getMemberList(MemberListType::AllMembersList());
     if (ml) ml->sort();
     writeMemberPages(ol);
   }
@@ -1050,7 +1054,7 @@ void FileDefImpl::writeMemberPages(OutputList &ol)
 
   for (const auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_documentationLists)
+    if (ml->listType().isDocumentation())
     {
       ml->writeDocumentationPage(ol,name(),this);
     }
@@ -1066,7 +1070,7 @@ void FileDefImpl::writeQuickMemberLinks(OutputList &ol,const MemberDef *currentM
   ol.writeString("      <div class=\"navtab\">\n");
   ol.writeString("        <table>\n");
 
-  MemberList *allMemberList = getMemberList(MemberListType_allMembersList);
+  MemberList *allMemberList = getMemberList(MemberListType::AllMembersList());
   if (allMemberList)
   {
     for (const auto &md : *allMemberList)
@@ -1189,12 +1193,14 @@ void FileDefImpl::writeSourceBody(OutputList &ol,[[maybe_unused]] ClangTUParser
       intf->parseCode(devNullList,QCString(),
                        fileToString(absFilePath(),TRUE,TRUE),
                        getLanguage(),
+                       Config_getBool(STRIP_CODE_COMMENTS),
                        FALSE,QCString(),this
                       );
     }
     intf->parseCode(codeOL,QCString(),
         fileToString(absFilePath(),filterSourceFiles,TRUE),
         getLanguage(),      // lang
+        Config_getBool(STRIP_CODE_COMMENTS),
         FALSE,              // isExampleBlock
         QCString(),         // exampleName
         this,               // fileDef
@@ -1238,6 +1244,7 @@ void FileDefImpl::parseSource([[maybe_unused]] ClangTUParser *clangParser)
             devNullList,QCString(),
             fileToString(absFilePath(),filterSourceFiles,TRUE),
             getLanguage(),
+            Config_getBool(STRIP_CODE_COMMENTS),
             FALSE,QCString(),this
            );
   }
@@ -1247,7 +1254,7 @@ void FileDefImpl::addMembersToMemberGroup()
 {
   for (auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_declarationLists)
+    if (ml->listType().isDeclaration())
     {
       ::addMembersToMemberGroup(ml.get(),&m_memberGroups,this);
     }
@@ -1270,7 +1277,7 @@ void FileDefImpl::insertMember(MemberDef *md)
   if (md->isHidden()) return;
   //printf("%s:FileDefImpl::insertMember(%s (=%p) list has %d elements)\n",
   //    qPrint(name()),qPrint(md->name()),md,allMemberList.count());
-  MemberList *allMemberList = getMemberList(MemberListType_allMembersList);
+  MemberList *allMemberList = getMemberList(MemberListType::AllMembersList());
   if (allMemberList && allMemberList->contains(md))
   {
     return;
@@ -1278,49 +1285,49 @@ void FileDefImpl::insertMember(MemberDef *md)
 
   if (allMemberList==nullptr)
   {
-    m_memberLists.emplace_back(std::make_unique<MemberList>(MemberListType_allMembersList,MemberListContainer::File));
+    m_memberLists.emplace_back(std::make_unique<MemberList>(MemberListType::AllMembersList(),MemberListContainer::File));
     allMemberList = m_memberLists.back().get();
   }
   allMemberList->push_back(md);
   switch (md->memberType())
   {
-    case MemberType_Property:
+    case MemberType::Property:
       if (md->getLanguage() == SrcLangExt::Python)
       {
-        addMemberToList(MemberListType_propertyMembers,md);
-        addMemberToList(MemberListType_properties,md);
+        addMemberToList(MemberListType::PropertyMembers(),md);
+        addMemberToList(MemberListType::Properties(),md);
         break;
       }
       //  fallthrough, explicitly no break here
-    case MemberType_Variable:
-      addMemberToList(MemberListType_decVarMembers,md);
-      addMemberToList(MemberListType_docVarMembers,md);
+    case MemberType::Variable:
+      addMemberToList(MemberListType::DecVarMembers(),md);
+      addMemberToList(MemberListType::DocVarMembers(),md);
       break;
-    case MemberType_Function:
-      addMemberToList(MemberListType_decFuncMembers,md);
-      addMemberToList(MemberListType_docFuncMembers,md);
+    case MemberType::Function:
+      addMemberToList(MemberListType::DecFuncMembers(),md);
+      addMemberToList(MemberListType::DocFuncMembers(),md);
       break;
-    case MemberType_Typedef:
-      addMemberToList(MemberListType_decTypedefMembers,md);
-      addMemberToList(MemberListType_docTypedefMembers,md);
+    case MemberType::Typedef:
+      addMemberToList(MemberListType::DecTypedefMembers(),md);
+      addMemberToList(MemberListType::DocTypedefMembers(),md);
       break;
-    case MemberType_Sequence:
-      addMemberToList(MemberListType_decSequenceMembers,md);
-      addMemberToList(MemberListType_docSequenceMembers,md);
+    case MemberType::Sequence:
+      addMemberToList(MemberListType::DecSequenceMembers(),md);
+      addMemberToList(MemberListType::DocSequenceMembers(),md);
       break;
-    case MemberType_Dictionary:
-      addMemberToList(MemberListType_decDictionaryMembers,md);
-      addMemberToList(MemberListType_docDictionaryMembers,md);
+    case MemberType::Dictionary:
+      addMemberToList(MemberListType::DecDictionaryMembers(),md);
+      addMemberToList(MemberListType::DocDictionaryMembers(),md);
       break;
-    case MemberType_Enumeration:
-      addMemberToList(MemberListType_decEnumMembers,md);
-      addMemberToList(MemberListType_docEnumMembers,md);
+    case MemberType::Enumeration:
+      addMemberToList(MemberListType::DecEnumMembers(),md);
+      addMemberToList(MemberListType::DocEnumMembers(),md);
       break;
-    case MemberType_EnumValue:    // enum values are shown inside their enums
+    case MemberType::EnumValue:    // enum values are shown inside their enums
       break;
-    case MemberType_Define:
-      addMemberToList(MemberListType_decDefineMembers,md);
-      addMemberToList(MemberListType_docDefineMembers,md);
+    case MemberType::Define:
+      addMemberToList(MemberListType::DecDefineMembers(),md);
+      addMemberToList(MemberListType::DocDefineMembers(),md);
       break;
     default:
        err("FileDefImpl::insertMembers(): "
@@ -1332,6 +1339,60 @@ void FileDefImpl::insertMember(MemberDef *md)
   //addMemberToGroup(md,groupId);
 }
 
+void FileDefImpl::removeMemberFromList(MemberListType lt,MemberDef *md)
+{
+  MemberList *ml = getMemberList(lt);
+  if (ml) ml->remove(md);
+}
+
+void FileDefImpl::removeMember(MemberDef *md)
+{
+  removeMemberFromList(MemberListType::AllMembersList(),md);
+  switch(md->memberType())
+  {
+    case MemberType::Property:
+      if (md->getLanguage() == SrcLangExt::Python)
+      {
+        removeMemberFromList(MemberListType::PropertyMembers(),md);
+        removeMemberFromList(MemberListType::Properties(),md);
+        break;
+      }
+      //  fallthrough, explicitly no break here
+    case MemberType::Variable:
+      removeMemberFromList(MemberListType::DecVarMembers(),md);
+      removeMemberFromList(MemberListType::DocVarMembers(),md);
+      break;
+    case MemberType::Function:
+      removeMemberFromList(MemberListType::DecFuncMembers(),md);
+      removeMemberFromList(MemberListType::DocFuncMembers(),md);
+      break;
+    case MemberType::Typedef:
+      removeMemberFromList(MemberListType::DecTypedefMembers(),md);
+      removeMemberFromList(MemberListType::DocTypedefMembers(),md);
+      break;
+    case MemberType::Sequence:
+      removeMemberFromList(MemberListType::DecSequenceMembers(),md);
+      removeMemberFromList(MemberListType::DocSequenceMembers(),md);
+      break;
+    case MemberType::Dictionary:
+      removeMemberFromList(MemberListType::DecDictionaryMembers(),md);
+      removeMemberFromList(MemberListType::DocDictionaryMembers(),md);
+      break;
+    case MemberType::Enumeration:
+      removeMemberFromList(MemberListType::DecEnumMembers(),md);
+      removeMemberFromList(MemberListType::DocEnumMembers(),md);
+      break;
+    case MemberType::EnumValue:    // enum values are shown inside their enums
+      break;
+    case MemberType::Define:
+      removeMemberFromList(MemberListType::DecDefineMembers(),md);
+      removeMemberFromList(MemberListType::DocDefineMembers(),md);
+      break;
+    default:
+      err("FileDefImpl::removeMember(): unexpected member remove in file!\n");
+  }
+}
+
 /*! Adds compound definition \a cd to the list of all compounds of this file */
 void FileDefImpl::insertClass(ClassDef *cd)
 {
@@ -1381,11 +1442,11 @@ const QCString &FileDefImpl::name() const
 
 void FileDefImpl::addSourceRef(int line,const Definition *d,const MemberDef *md)
 {
-  //printf("FileDefImpl::addSourceDef(%d,%p,%p)\n",line,d,md);
+  //printf("FileDefImpl::addSourceRef(%d,%s,%s)\n",line,qPrint(d?d->name():"nullptr"),qPrint(md?md->name():"nullptr"));
   if (d)
   {
-    m_srcDefMap.insert(std::make_pair(line,d));
-    if (md) m_srcMemberMap.insert(std::make_pair(line,md));
+    m_srcDefMap.emplace(line,d);
+    if (md) m_srcMemberMap.emplace(line,md);
     //printf("Adding member %s with anchor %s at line %d to file %s\n",
     //    md?qPrint(md->name()):"<none>",md?qPrint(md->anchor()):"<none>",line,qPrint(name()));
   }
@@ -1418,19 +1479,19 @@ const LinkedRefMap<NamespaceDef> &FileDefImpl::getUsedNamespaces() const
   return m_usingDirList;
 }
 
-void FileDefImpl::addUsingDeclaration(ClassDef *cd)
+void FileDefImpl::addUsingDeclaration(const Definition *d)
 {
-  m_usingDeclList.add(cd->qualifiedName(),cd);
+  m_usingDeclList.add(d->qualifiedName(),d);
 }
 
 void FileDefImpl::addIncludeDependency(const FileDef *fd,const QCString &incName,IncludeKind kind)
 {
-  //printf("FileDefImpl::addIncludeDependency(%p,%s,%d)\n",fd,incName,local);
+  //printf("FileDefImpl::addIncludeDependency(%p,%s,%d)\n",(void*)fd,qPrint(incName),kind);
   QCString iName = fd ? fd->absFilePath() : incName;
   if (!iName.isEmpty() && m_includeMap.find(iName.str())==m_includeMap.end())
   {
     m_includeList.emplace_back(fd,incName,kind);
-    m_includeMap.insert(std::make_pair(iName.str(),&m_includeList.back()));
+    m_includeMap.emplace(iName.str(),&m_includeList.back());
   }
 }
 
@@ -1468,11 +1529,11 @@ void FileDefImpl::addIncludedUsingDirectives(FileDefSet &visitedFiles)
             m_usingDirList.prepend(nd->qualifiedName(),nd);
           }
           // add using declarations
-          auto  udl = ii.fileDef->getUsedClasses();
+          auto  udl = ii.fileDef->getUsedDefinitions();
           for (auto it = udl.rbegin(); it!=udl.rend(); ++it)
           {
-            auto *cd = *it;
-            m_usingDeclList.prepend(cd->qualifiedName(),cd);
+            auto *d = *it;
+            m_usingDeclList.prepend(d->qualifiedName(),d);
           }
         }
       }
@@ -1489,7 +1550,7 @@ void FileDefImpl::addIncludedByDependency(const FileDef *fd,const QCString &incN
   if (!iName.isEmpty() && m_includedByMap.find(iName.str())==m_includedByMap.end())
   {
     m_includedByList.emplace_back(fd,incName,kind);
-    m_includedByMap.insert(std::make_pair(iName.str(),&m_includedByList.back()));
+    m_includedByMap.emplace(iName.str(),&m_includedByList.back());
   }
 }
 
@@ -1529,7 +1590,7 @@ void FileDefImpl::addListReferences()
   }
   for (auto &ml : m_memberLists)
   {
-    if (ml->listType()&MemberListType_documentationLists)
+    if (ml->listType().isDocumentation())
     {
       ml->addListReferences(this);
     }
@@ -1559,9 +1620,9 @@ void FileDefImpl::combineUsingRelations()
       addUsingDirective(und);
     }
     // add used classes of namespace nd to this namespace
-    for (const auto &ucd : nd->getUsedClasses())
+    for (const auto &ud : nd->getUsedDefinitions())
     {
-      addUsingDeclaration(ucd);
+      addUsingDeclaration(ud);
     }
   }
 }
@@ -1639,10 +1700,10 @@ void FileDefImpl::addMemberToList(MemberListType lt,MemberDef *md)
   bool sortMemberDocs = Config_getBool(SORT_MEMBER_DOCS);
   const auto &ml = m_memberLists.get(lt,MemberListContainer::File);
   ml->setNeedsSorting(
-       ((ml->listType()&MemberListType_declarationLists) && sortBriefDocs) ||
-       ((ml->listType()&MemberListType_documentationLists) && sortMemberDocs));
+       (ml->listType().isDeclaration() && sortBriefDocs) ||
+       (ml->listType().isDocumentation() && sortMemberDocs));
   ml->push_back(md);
-  if (ml->listType()&MemberListType_declarationLists)
+  if (ml->listType().isDeclaration())
   {
     MemberDefMutable *mdm = toMemberDefMutable(md);
     if (mdm)
@@ -1791,13 +1852,13 @@ void FileDefImpl::countMembers()
 
 int FileDefImpl::numDocMembers() const
 {
-  MemberList *ml = getMemberList(MemberListType_allMembersList);
+  MemberList *ml = getMemberList(MemberListType::AllMembersList());
   return ml ? ml->numDocMembers() : 0;
 }
 
 int FileDefImpl::numDecMembers() const
 {
-  MemberList *ml = getMemberList(MemberListType_allMembersList);
+  MemberList *ml = getMemberList(MemberListType::AllMembersList());
   return ml ? ml->numDecMembers() : 0;
 }
 
